/* usbservo - controlling servo motors over USB, simple project for ATtiny2313,
   Pavel, ppetrovic at acm.org, robotika.sk,

   Based on PowerSwitch example of V-USB - see readme.txt in usbdrv/ for details and licence! */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>

#include "usbdrv.h"
#include "oddebug.h"
#include <util/delay.h>

#define SET_BIT(PORT, BIT)  ((PORT) |= (1<<BIT))
#define CLR_BIT(PORT, BIT)  ((PORT) &= ~(1<<BIT))

static unsigned int posA, posB;
static unsigned int motorA_neutral, motorB_neutral;

/* initialize timers */
void servo_init()
{
  posA = motorA_neutral;
  posB = motorB_neutral;
  cli();
  OCR1A = motorA_neutral;
  OCR1B = motorB_neutral;
  ICR1 = 29999;
  TIMSK = 0;
  TIFR = 0;
  TCCR1B = 0b00011010;
  TCCR1A = 0b10100010;
  TCCR1C = 0;
  sei();
  // set the OC1A/OC1B as output pins
  DDRB = 8 + 16;
  PORTB = 8 + 16;
}


USB_PUBLIC uchar usbFunctionSetup(uchar data[8])
{
usbRequest_t    *rq = (void *)data;
static uchar    replyBuf[4];

    usbMsgPtr = replyBuf;
    if(rq->bRequest == 0)  /* ECHO */
	{
        replyBuf[0] = rq->wValue.bytes[0];
        replyBuf[1] = rq->wValue.bytes[1];
        return 2;
    }
    else if(rq->bRequest == 1)  /* GET_STATUS -> result = 2 bytes */
    {
        replyBuf[0] = 06;
        replyBuf[1] = 21;
        return 2;
    }
    else if(rq->bRequest == 2)  /* GET_POSITION -> result = 4 bytes */
    {
        replyBuf[0] = posA >> 8;
        replyBuf[1] = posA & 255;
		replyBuf[2] = posB >> 8;
		replyBuf[3] = posB & 255;
        return 4;
    }
	else if(rq->bRequest == 3) /* motorA,motorB increase by (signed char)X,Y */
	{
	    posA += (char)rq->wValue.bytes[0];
		posB += (char)rq->wValue.bytes[1];
		cli();
		OCR1A = posA;
		OCR1B = posB;
		sei();
	}
	else if(rq->bRequest == 4) /* motorA set to (unsigned int)X */
	{
	   posA = ((unsigned char)rq->wValue.bytes[0]) + ((unsigned char)rq->wValue.bytes[1] << 8);
	   cli();
	   OCR1A = posA;
	   sei();
	}
	else if(rq->bRequest == 5) /* motorB set to (unsigned int)X */
	{
	   posB = ((unsigned char)rq->wValue.bytes[0]) + ((unsigned char)rq->wValue.bytes[1] << 8);
	   cli();
	   OCR1B = posB;
	   sei();
	}
    return 0;
}

int main(void)
{
    motorA_neutral = 2120;
	motorB_neutral = 2120;
	uchar   i;

    odDebugInit();
/* We fake an USB disconnect by pulling D+ and D- to 0 during reset. This is
 * necessary if we had a watchdog reset or brownout reset to notify the host
 * that it should re-enumerate the device. Otherwise the host's and device's
 * concept of the device-ID would be out of sync.
 */
    DDRD = ~USBMASK;    /* set all pins as outputs except USB */
    PORTD = 0;          /* no pullups on USB pins */
	servo_init();

    PORTD |= 64;   /* turn on status LED */
	for (i = 0; i < 10; i++) _delay_ms(2);
	PORTD |= 64;   /* turn off status LED */
	for (i = 0; i < 10; i++) _delay_ms(2);
    PORTD |= 64;   /* turn on status LED */

    wdt_enable(WDTO_1S);
    usbDeviceDisconnect();  /* enforce re-enumeration */
    i = 0;
    while(--i){         /* fake USB disconnect for > 500 ms */
        wdt_reset();
        _delay_ms(2);
    }
    usbDeviceConnect();
    usbInit();
    sei();

    for(;;){    /* main event loop */
        wdt_reset();
        usbPoll();
    }
    return 0;
}
